\\ Copyright 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.


default(recover,0);
default(strictargs,1);
read("alternate-defines.gp");

\\ Philip Lafrance, Narad Rampersad, Randy Yee, "Some Properties of a
\\ Rudin-Shapiro-Like Sequence", arxiv 1408.2277
\\
\\ A246016 = in(n)

\\-----------------------------------------------------------------------------
Count10Pairs(n) = sum(i=0,if(n,logint(n,2)), bittest(n,i+1) && !bittest(n,i));
\\ vector(20,n,Count10Pairs(n))
\\ 1, 0, 1, 1, 1, 0, 1, 1, 2, 1, 1, 1, 1, 0, 1, 1, 2, 1, 2
\\ A033264

\\ vector(20,n,(-1)^Count10Pairs(n))
\\ not in OEIS: 1,-1,1,-1,-1,-1,1,-1,-1,1,-1,-1,-1,-1,1,-1,-1,1,-1,1

Count01Pairs(n) = sum(i=0,if(n,logint(n,2)), !bittest(n,i+1) && bittest(n,i));
\\ vector(20,n,(-1)^Count01Pairs(n))
\\ not in OEIS: -1,-1,-1,-1,1,-1,-1,-1,1,1,1,-1,1,-1,-1,-1,1,1,1,1

vector(2^10,n,(-1)^(Count01Pairs(n) + Count10Pairs(n))) == \
vector(2^10,n,(-1)^n)  || error();
\\ A033999 (-1)^n


\\-----------------------------------------------------------------------------
CountScattered10(n) =
{
  my(ret=0,zeros=0);
  for(i=0,if(n,logint(n,2)),
     if(bittest(n,i), ret += zeros, zeros++));
  ret;
}
\\ vector(20,n,CountScattered10(n))
\\ 0, 1, 0, 2, 1, 2, 0, 3, 2, 3, 1, 4, 2, 3, 0, 4, 3, 4, 2, 5
\\ A055941
\\ inversion = occurrence of ba in a word where b>a

vector(2^12,n,n--; CountScattered10(n) % 2) == \
vector(2^12,n,n--; bittest(PmOneBits(n),1))  || error();

A055941_compact(n) = \
  my(ret=0,z=0); for(i=0,if(n,logint(n,2)), if(bittest(n,i), ret+=z, z++)); ret;

vector(2^12,n,n--; CountScattered10(n)) == \
vector(2^12,n,n--; A055941_compact(n))  || error();

sqrtint_ceil(n) = sqrtint(n-1)+1;
vector(100,n, sqrtint_ceil(n)) == \
vector(100,n, sqrtint(n) + !issquare(n))  || error();

{
  \\ CountScattered10 index of new highs
  \\ 2,4,8,12,20,24,40,48,56,88,104,112,176,208,224,240,368,432,464,480,736,864,928,960,992,
  \\ A243112
  \\
  \\ not in OEIS: 10,100,1000,1100,10100,11000,101000,110000,111000,1011000,1101000,1110000,10110000,11010000,11100000,11110000,101110000,110110000,111010000,111100000,1011100000,1101100000,1110100000,1111000000,1111100000,
  \\
  \\   .1111100000,  c=25    5,0, 5
  \\   10111100000,  c=26    1,4
  \\   11011100000,  c=27
  \\   11101100000,  c=28
  \\   11110100000,  c=29    4,1, 5
  \\  .11111000000,  c=30    0,5, 6
  \\  101111000000,  c=31    1,4, 6      7 + 6+6+6+6   = 31     length 12
  \\  110111000000,  c=32
  \\  111011000000,  c=33
  \\  111101000000,  c=34
  \\  111110000000,  c=35         6
  \\  111111000000,  c=36         6
  \\ 1011111000000,  c=37
  \\
  \\ t ones, z zeros
  \\ high term z, total length t+z
  \\ t terms
  \\ each t adds at most z
  \\ so need  t >= floor(k/z)  of them
  \\ l = t+z >= z + floor(k/z)
  \\ l >= z + k/z - r/z
  \\ l+1 > z + k/z                   hyperbola sheared
  \\ (x + k/x)' == (x^2 - k)/x^2     zero at x^2 = k
  \\ ploth(x=.1,5, x+4/x)
  \\ min at z=sqrt(k)                num zeros = ceil(sqrt(k))
  \\ t = floor(k/z)
  \\ then put fewest high 1s as possible, so k/z
  \\
  my(m=0);
  for(n=0,0^14,
     my(c=CountScattered10(n));
     if(c>m,
        m=c;
        print(to_binary(n),",  c="c)));
}

CountScattered10_first(k) =
{
   my(t=sqrtint(k),
      r=k-t^2,
      lo=t);
  if(r>=t, r-=t;lo++);
  (2^(t+1) - 2^(t-r) - 1)*2^lo;

  my(t=sqrtint(k), r=k-t^2);
  if(r<t, (2^(t+1) - 2^(t-r) - 1)*2^t,
          (2^(t+1) - 2^(2*t-r) - 1)*2^(t+1));

  my(t=sqrtint(k), r=k-t^2, h=2*t);
  if(r>=t, r-=t;t++;h++);
  2<<h - 1<<(h-r) - 1<<t;
}
{
  my(m=0);
  for(n=0,0* 2^12,
     my(k=CountScattered10(n));
     if(k>m,
        m=k;

   my(t=sqrtint(k),
      r=k-t^2,
      p=r,
      lo=t);
  if(r<t, , lo++;r-=t);

        printf("%14s  k=%d  t=%d,r=%d,lo=%d,p=%d\n",
               to_binary(n), k, t,r,lo,p);
        my(got=CountScattered10_first(k));
        printf("%14s  %s\n", to_binary(got),if(got==n,""," *******"));
        print()));
}

{
  my(m=0);
  for(n=0,2^14,
     my(k=CountScattered10(n));
     if(k>m, m=k; n==CountScattered10_first(k) || error("m=",m)));
}

\\ vector(20,k,k+=3; my(t=sqrtint(k),r=k-t^2,lo=t); if(r>=t, r-=t;lo++);  lo)
\\ 2, 2, 3, 3, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5
\\ A000194  n appears n times

\\vector(20,k,k+=3; my(t=sqrtint(k),r=k-t^2,lo=t); if(r>=t, r-=t;lo++);  t-r)
\\ not in OEIS: 2, 1, 2, 1, 0, 3, 2, 1, 3, 2, 1, 0, 4, 3, 2, 1, 4, 3, 2, 1

\\ vector(20,k,k+=3; my(t=sqrtint(k),r=k-t^2,lo=t); if(r>=t, r-=t;lo++);  CountScattered10_first(k)/2^lo)
\\ not in OEIS: 3, 5, 3, 5, 6, 7, 11, 13, 7, 11, 13, 14, 15, 23, 27, 29, 15, 23, 27, 29


\\-----------------------------------------------------------------------------
in(n) = (-1)^CountScattered10(n);
\\ vector(20,n, in(n))
\\ 1, -1, 1, 1, -1, 1, 1, -1, 1, -1, -1, 1, 1, -1, 1, 1, -1, 1, 1, -1
\\ A246016
\\ inversion = occurrence of ba in a word where b>a

\\ vector(#A189301_samples,n,n--; in(n)>=0)  - \
\\ 1, 0, 1, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0
\\ not A189301, though close

\\ vector(20,n, in(n)<=0)
\\ not in OEIS: 0, 1, 0, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1
\\ not A242718 middle match
\\ not A286691 middle match



\\ morphism A -> AB, B -> CA, C -> BD, D -> DC
\\ cf GRS   A -> AB, B -> AC, C -> DB, D -> DC

state_table = {[0,1;
                2,0;
                1,3;
                3,2]};
st(n) =
{
  my(v=binary(n),state=0);
  for(i=1,#v, state=state_table[state+1,v[i]+1]);
  state;
}
vector(1000,n,n--; (-1)^(st(n)>=2)) == \
vector(1000,n,n--; in(n)) || error();
\\ vector(20,n, st(n))
\\ not in OEIS: 1, 2, 0, 1, 3, 0, 1, 2, 0, 3, 2, 0, 1, 2, 0, 1, 3, 0, 1, 3
\\ vector(20,n, st(n)+1)
\\ not in OEIS: 2, 3, 1, 2, 4, 1, 2, 3, 1, 4, 3, 1, 2, 3, 1, 2, 4, 1, 2, 4
\\ vector(20,n, st(2*n))
\\ not in OEIS: 2, 1, 0, 2, 3, 0, 2, 1, 0, 3, 1, 0, 2, 1, 0, 2, 3, 0, 2, 3
\\ vector(20,n, st(2*n+1))
\\ not in OEIS: 0, 3, 1, 0, 2, 1, 0, 3, 1, 2, 3, 1, 0, 3, 1, 0, 2, 1, 0, 2
\\ vector(20,n, [0,1,3,2][st(n)+1])
\\ not in OEIS: 1, 3, 0, 1, 2, 0, 1, 3, 0, 2, 3, 0, 1, 3, 0, 1, 2, 0, 1, 2

\\ states 1,2 are odd 1s above, so that each 0 is a flip
vector(1000,n,n--; my(s=st(n)); s==1||s==2) == \
vector(1000,n,n--; hammingweight(n)%2==1) || error();

\\ states 0,3 are even 1s above, so that each 0 is no change
vector(1000,n,n--; my(s=st(n)); s==0||s==3) == \
vector(1000,n,n--; hammingweight(n)%2==0) || error();

\\ 0 1  2  3                       0 1  3  2
\\ 0 1  1  0  parity 1s            0 1  0  1  parity 1s
\\ 1 1 -1 -1  result               1 1 -1 -1  result


{ my(table=[2,1; 1,3; 4,2; 3,4]);
  in_by_state(n) = my(state=1);
    for(i=0,if(n,logint(n,2)), state=table[state,bittest(n,i)+1]);
    [1,1,-1,-1][state];
}
table == 'table || error();  \\ not a global
vector(1000,n,n--; in_by_state(n)) == \
vector(1000,n,n--; in(n)) || error();

{ my(table=[3,4; 4,2; 1,3; 2,1]);
  in_by_state(n) = my(state=2);
    for(i=0,if(n,logint(n,2)), state=table[state,bittest(n,i)+1]);
    (-1)^state;
}
table == 'table || error();  \\ not a global
vector(1000,n,n--; in_by_state(n)) == \
vector(1000,n,n--; in(n)) || error();

\\               1     0     1
\\         0s   <->   <->   <->   0s
\\ states     A     B     C     D
\\           +1    -1    -1    +1
\\           +1    +1    -1    -1
\\            0     1     3     2    PmOneBits % 4
vector(2^12,n,n--; st(n)) == \
vector(2^12,n,n--; [0,1,3,2][PmOneBits(n)%4+1])  || error();


\\-----------------------------------------------------------------------------
\\ skip low 1s, each low zero flip if hammingweight above it odd
vector(1000,n,n--; in(2*n)) == \
vector(1000,n,n--; in(n)*(-1)^hammingweight(n)) || error();
vector(1000,n,n--; in(2*n+1)) == \
vector(1000,n,n--; in(n)) || error();

\\ low 00 ignore
\\     1  ignore
\\     10 flip and keep 0  (so another 1 flips again)
\\     11 ignore
vector(1000,n,n--; in(4*n)) == \
vector(1000,n,n--; in(n)) || error();
vector(1000,n,n--; in(4*n+1)) == \
vector(1000,n,n--; in(2*n)) || error();
vector(1000,n,n--;   in(4*n+2)) == \
vector(1000,n,n--; - in(2*n)) || error();
vector(1000,n,n--; in(4*n+3)) == \
vector(1000,n,n--; in(n)) || error();

\\ vector(20,n, in(n))
\\ vector(20,n, (-1)^hammingweight(bitand(n,bitneg(n)<<1)))
\\ vector(20,n, my(v=digits(n,4)); (-1)^sum(i=1,#v, v[i]==1||v[i]==2))
\\ vector(20,n, bittest(hammingweight(n),2))

\\ my(len=2^16+1, zv=vector(len), z=0,d=1); \
\\ for(n=1,len, zv[n]=z; z+=d; d*=I^in(n)); plothraw(real(zv),imag(zv),1);

\\ A269723 certain complementing
\\ is parity of count odd position 1-bits
\\ parity of count base4 digits 2,3
A269723_vector(len) =
{
  my(A=[0]);
  while(#A<len,
        my(B=apply(n->1-n,A));
        A=concat([A,A,B,B]));
  A[1..len];
}
my(v=OEIS_samples("A269723")); A269723_vector(#v) == v  || error();

vector(2^14,n,n--; PmOneBits(n) % 2) == \
vector(2^14,n,n--; hammingweight(n) % 2)  || error();

vector(2^14,n,n--; bittest(PmOneBits(n) - hammingweight(n), 1)) == \
A269723_vector(2^14)  || error();
\\ 0,0,1,1,0,0,1,1,1,1,0,0,1,1,0,0,0,0,1,1,0,0,1,1,1,1

vector(15,n, (hammingweight(n) - PmOneBits(n)) / 2)
\\ 0, 1, 1, 0, 0, 1, 1, 1, 1, 2, 2, 1, 1, 2, 2


\\-----------------------------------------------------------------------------
vector(20,n, GRS(Gray(n)))
\\ not in OEIS: 1, -1, 1, -1, 1, 1, 1, -1, -1, -1, 1, 1, -1, 1, 1, -1, -1, 1

\\ Gray = A003188
\\ UnGray = A006068
vector(2^10,n, in(n)) == \
vector(2^10,n, GRS(UnGray(n)))  || error();
vector(2^10,n, in(Gray(n))) == \
vector(2^10,n, GRS(n))  || error();

vector(2^10,n, dir(n)) == \
vector(2^10,n, PmOneBits(Gray(n)))  || error();
vector(2^10,n, GRS(n)) == \
vector(2^10,n, (-1)^(dir(n)\2))  || error();
vector(2^10,n, GRS(n)) == \
vector(2^10,n, 1-bitand(2,dir(n)))  || error();
vector(2^10,n, in(n)) == \
vector(2^10,n, 1-bitand(2,dir(UnGray(n))))  || error();
vector(2^10,n, in(n)) == \
vector(2^10,n, 1-bitand(2,PmOneBits(Gray(UnGray(n)))))  || error();
vector(2^10,n, in(n)) == \
vector(2^10,n, (-1)^bittest(PmOneBits(n),1))  || error();
vector(2^10,n, in(n)) == \
vector(2^10,n, 1-bitand(2,PmOneBits(n)))  || error();

vector(2^10,n, in(n)) == \
vector(2^10,n, (-1)^(sum(i=0,if(n,logint(n,2)), (-1)^i*bittest(n,i))\2)) \
  || error();
vector(2^10,n, in(n)) == \
vector(2^10,n, 1-bitand(2, sum(i=0,if(n,logint(n,2)), (-1)^i*bittest(n,i)))) \
  || error();

\\       v--- low from bit
\\  0 = 00                                     hammingweight
\\  1 = 01                                   + 2 at each odd i
\\ -1 = 11
\\      ^-- high from i, when bit==1

vector(2^10,n, in(n)) == \
vector(2^10,n, \
  1-bitand(2, sum(i=0,if(n,logint(n,2)), if(bittest(n,i), 1+(i<<1)))) \
  ) || error();

\\ +2  for odd 1s
\\ +1  for odd+even 1s

vector(2^10,n, in(n)) == \
vector(2^10,n, 1-bitand(2,dir(UnGray(n))))  || error();

vector(2^10,n, in(n)) == \
vector(2^10,n, \
  my(k=dir_mod_4_Arndt_k(n)); \
  1-bitand(2, CountOneBits(bitxor(n, dir_mod_4_Arndt_flip(k))))  \
  ) || error();

vector(2^10,n, GRS(n)) == \
vector(2^10,n, \
  my(k=dir_mod_4_Arndt_k(n)); \
  1-bitand(2, CountOneBits(bitxor(Gray(n), dir_mod_4_Arndt_flip(k))))  \
  ) || error();


\\ my(len=2^8+1, zv=vector(len), z=0,d=1); \
\\ for(n=1,len, zv[n]=z; z+=in(n)*if(n%2,1,I)); \
\\ plothraw(real(zv),imag(zv),1);

\\ my(len=2^4+1, zv=vector(len)); \
\\ for(n=1,len, zv[n]=in(n)*if(n%2,1,I)); /* dz steps */ \
\\ my(z=0,l=List([])); \
\\ for(n=1,len, listput(l,z+.2*zv[n]); z+=zv[n]; listput(l,z-.2*zv[n])); \
\\ zv=Vec(l); \
\\ plothraw(real(zv),imag(zv),1);

\\ \\ PmOneBits as direction function is Koch curve, or whatever angle
\\ my(len=2^8+1, zv=vector(len), shorten=.0); \
\\ for(n=1,len, zv[n]=exp(I*Pi/3*PmOneBits(n-1))); /* dz steps */ \
\\ my(z=0,l=List([])); \
\\ for(n=1,len, listput(l,z+shorten*zv[n]); z+=zv[n]; listput(l,z-shorten*zv[n])); \
\\ zv=Vec(l); \
\\ plothraw(real(zv),imag(zv),1);

reverse_and_flip(n) =
{
  my(v=binary(n));
  fromdigits(Vecrev(apply(bit->1-bit,v)), 2);
}
\\ vector(20,n, reverse_and_flip(n))
\\ 0, 2, 0, 6, 2, 4, 0, 14, 6, 10, 2, 12, 4, 8, 0, 30, 14, 22, 6, 26
\\ A036044 binary complement reverse

\\ it reckons n=0 as one bit to complement
A036044(n) = \
  if(n==0,1, my(v=binary(n)); fromdigits(Vecrev(apply(bit->1-bit,v)), 2));
my(v=OEIS_samples("A036044")); vector(#v,n,n--; A036044(n)) == v  || error();

vector(2^10,n, in(n)) == \
vector(2^10,n, in(reverse_and_flip(n)))  || error();
vector(2^10,n,n--; in(n)) == \
vector(2^10,n,n--; in(A036044(n)))  || error();




\\-----------------------------------------------------------------------------
