#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'chamfer';
use MyPrintwrap ':all';


my $k = 3;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyK}{$k}\n";
my $path = Math::PlanePath::AlternatePaper->new;

my $n_lo_GRS = 4**$k;
my $n_hi_GRS = 4**($k+1) - 1;  # inclusive
my $n_lo = $n_lo_GRS + 1;  # to plot point(n+1)
my $n_hi = $n_hi_GRS + 1;

{
  my ($x,$y) = $path->n_to_xy($n_lo);
  print "  \\node (start) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  print "  \\node (end) at ($x,$y) [my dot];\n";
}
$Printwrap = 2;
printwrap("\\draw (start)");
$Printwrap += 2;
foreach my $n ($n_lo+1 .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);

  my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
  my ($tdx,$tdy) = $path->n_to_dxdy($n);
  my $sx = $x . chamfer(-$sdx);
  my $sy = $y . chamfer(-$sdy);
  my $tx = $x . chamfer($tdx);
  my $ty = $y . chamfer($tdy);
  printwrap("--($sx,$sy)");
  printwrap("--($tx,$ty)");
}
printwrap("--(end);\n");

print "  % end generated\n";
exit 0;
