#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'chamfer','xy_rotate_minus90','div';
use MyPrintwrap ':all';


my $level = 5;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::AlternatePaper->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my @joins;
my ($prev_x,$prev_y);
for (my $n = 0; $n < $n_hi; $n+=2) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+2);
  my $rev = (($n >> 1) & 1);
  if ($rev) {
    ($x1,$y1, $x2,$y2) = ($x2,$y2, $x1,$y1);
  }
  my $dx = $x2 - $x1;
  my $dy = $y2 - $y1;
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
  my ($fx,$fy) = div($rx,$ry);
  my $cx = $x2;
  my $cy = $y2;

  $x1 .= chamfer($dx,'\\MyStart');
  $y1 .= chamfer($dy,'\\MyStart');
  $x2 .= chamfer(-$dx,'\\MyEnd');
  $y2 .= chamfer(-$dy,'\\MyEnd');

  $x1 .= chamfer($rx,'\\MyGap');
  $y1 .= chamfer($ry,'\\MyGap');
  $x2 .= chamfer($rx,'\\MyGap');
  $y2 .= chamfer($ry,'\\MyGap');

  $Printwrap = 2;
  printwrap(" \\draw[my segment] ($x1,$y1)");
  $Printwrap = 4;
  printwrap("--($x2,$y2);");

  $cx .= chamfer($fx,'\\MyCentre');
  $cy .= chamfer($fy,'\\MyCentre');
  my $num = $n/2;
  printwrap(" \\node at ($cx,$cy) {$num};");

  my ($start_x,$start_y) = (!$rev ? ($x1,$y1) : ($x2,$y2));
  my ($end_x,$end_y)     = (!$rev ? ($x2,$y2) : ($x1,$y1));
  if (defined $prev_x) {
    push @joins, " ($prev_x,$prev_y)", "--($start_x,$start_y)";
  }
  ($prev_x,$prev_y) = ($end_x,$end_y);
}
printwrap_newline();

$Printwrap = 2;
printwrap(" \\draw[my join]");
$Printwrap = 4;
foreach my $str (@joins) {
  printwrap($str);
}
printwrap(";\n");

print "  % end generated\n";
exit 0;
