#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate '@dir4_to_dx','@dir4_to_dy',
  'xy_rotate_minus90','xy_rotate_plus90','chamfer','turn';
use MyLineSegments;
use MyPrintwrap ':all';


my $w = 6;
my $h = 4;
if (@ARGV) { $w = shift @ARGV; $h = shift @ARGV; }

print "    % generated by tools/$FindBin::Script $w $h\n";
print "    \\newcommand\\MyW{$w} \\newcommand\\MyXmax{",$w-1,"}\n";
print "    \\newcommand\\MyH{$h} \\newcommand\\MyYmax{",$h-1,"}\n";

my $segments = MyLineSegments->new;
my %vertices;
foreach my $y (0 .. $h-1) {
  foreach my $x (-$y .. $w-1-$y) {
    {
      my $y2 = $y-1;
      if ($vertices{"$x,$y2"}) {
        $segments->add ($x,$y, $x,$y2);
      }
    }
    {
      my $x2 = $x-1;
      if ($vertices{"$x2,$y"}) {
        $segments->add ($x,$y, $x2,$y);
      }
    }
    $vertices{"$x,$y"} = 1;
  }
}
$Printwrap = 4;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

print "    \\node (p0-0) at (0,0) [my dot];\n";
delete $vertices{'0,0'};

$Printwrap = 4;
printwrap("\\path foreach \\p in {");
$Printwrap += 2;
foreach my $xy (sort keys %vertices) {
  printwrap("($xy),");
}
printwrap("} ");
printwrap("{ \\p node[my dot] };\n");

print "    % end generated\n";
exit 0;
